package io.xjar.key;

import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.security.SecureRandom;

public class XSecureRandom extends SecureRandom
{
    private byte[] _data;
    private int _index;
    private int _intPad;
    
    public XSecureRandom(final byte[] value) {
        this(false, new byte[][] { value });
    }
    
    public XSecureRandom(final byte[][] values) {
        this(false, values);
    }
    
    public XSecureRandom(final boolean intPad, final byte[] value) {
        this(intPad, new byte[][] { value });
    }
    
    public XSecureRandom(final boolean intPad, final byte[][] values) {
        final ByteArrayOutputStream bos = new ByteArrayOutputStream();
        for (int i = 0; i != values.length; ++i) {
            try {
                bos.write(values[i]);
            }
            catch (final IOException e) {
                throw new IllegalArgumentException("can't save value array.");
            }
        }
        this._data = bos.toByteArray();
        if (intPad) {
            this._intPad = this._data.length % 4;
        }
    }
    
    @Override
    public void nextBytes(final byte[] bytes) {
        System.arraycopy(this._data, this._index, bytes, 0, bytes.length);
        this._index += bytes.length;
    }
    
    @Override
    public byte[] generateSeed(final int numBytes) {
        final byte[] bytes = new byte[numBytes];
        this.nextBytes(bytes);
        return bytes;
    }
    
    @Override
    public int nextInt() {
        int val = 0;
        val |= this.nextValue() << 24;
        val |= this.nextValue() << 16;
        if (this._intPad == 2) {
            --this._intPad;
        }
        else {
            val |= this.nextValue() << 8;
        }
        if (this._intPad == 1) {
            --this._intPad;
        }
        else {
            val |= this.nextValue();
        }
        return val;
    }
    
    @Override
    public long nextLong() {
        long val = 0L;
        val |= (long)this.nextValue() << 56;
        val |= (long)this.nextValue() << 48;
        val |= (long)this.nextValue() << 40;
        val |= (long)this.nextValue() << 32;
        val |= (long)this.nextValue() << 24;
        val |= (long)this.nextValue() << 16;
        val |= (long)this.nextValue() << 8;
        val |= this.nextValue();
        return val;
    }
    
    public boolean isExhausted() {
        return this._index == this._data.length;
    }
    
    private int nextValue() {
        return this._data[this._index++] & 0xFF;
    }
}
